<!--- 
	csw-icontextblock-generic-rh.cfm

	This 'generic' render handler render handler implements the 'getGenericFieldMappingsArray' function below to describe
	the available field that can be mapped.

	The logic above the function, renders an image with a caption above it, and optionally renders a description
	and link.

	Note that for 'generic' render handlers the mapped data, if any, will always be passed in the following structure:
		Attributes.ElementInfo.ElementData.PropertyValues
	This is an array of one or more items, with a sub-structure named 'values'.
	Values is a structure with members as defined by the getGenericFieldMappingsArray() function.

	This generic render handler assumes these fields:
		- Title - Required
		- Subtitle - optional
		- Description - Optional
		- IconClass - Required (string or struct)
		- LinkURL - Optional - (string or struct)
		- LinkText - Optional - (string)
		
	Properties
		- Layout - Required
				stacked=cswLayout-Stacked
				stacked with line below title=cswLayout-StackedWithLine
				Icon Title On Same Line With Text Below=cswLayout-IconTitleOnSameLine
				Icon Next To Title And Text=cswLayout-IconNextTo
				Two Column Wrap=cswLayout-TwoColumnWrap				

	
	Version: [2018-01-16 15:22]		
--->


<cfscript>
	// Create the Icon Textblock Component 
	csWidget = CreateObject('component','csw-icontextblock');
	
	// load necessary resources
	csWidget.loadResources();

	// exit if all we're doing is detecting required resources
	if( Request.RenderState.RenderMode EQ "getResources" )
		exit;

	if( NOT structKeyExists(attributes,'elementinfo') )
		exit;
		
	// Load the Properties from the metadata form		
	props = csWidget.loadProperties(Attributes.ElementInfo,Variables.Caller);	
	
	// Make sure we have Props data
	if( NOT csWidget.hasProperties(props) )
		exit;	

	// Get the element Data and Render the items
	if( StructKeyExists(Attributes.ElementInfo.ElementData, 'PropertyValues') )
		csWidget.getAndRenderItems( Attributes.ElementInfo.ElementData, props );
</cfscript>


<!--- ////////////////////////////////////////////////////////////////////////// --->
<!--- //////////////////   GENERIC RENDER HANDLER FUNCTIONS   ////////////////// --->
<!--- ////////////////////////////////////////////////////////////////////////// --->

<!---
	This is the function that is called by CommonSpot when this render handler is registred, allowing teh user to
	map the fields of the source element to this render handler.

	Here we expose build a structure describing the four (4) fields that can be mapped. Two (2) of the fields are marked
	as required. The other two are optional.
		- Title - Required
		- IconClass - Required (string or csImage structure)
		- Subtitle - optional
		- Description - optional
		- LinkURL  ((string or csExtURL structure)
		- LinkText (string)

	Each fieldMappingArray item is a structure with the following 3 fields:
		- FieldName
		- Description
		- Required
--->
<cffunction name="getGenericFieldMappingsArray" returntype="array" access="private" output="no">
	<cfscript>
		var fieldMappingArray = ArrayNew(1);
		var fieldDetailsStruct = StructNew();

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'IconClass';
		fieldDetailsStruct.Description = 'The class name of the Icon to display for the Icon Text Block.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Title';
		fieldDetailsStruct.Description = 'The Title for the Icon Text Block.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Subtitle';
		fieldDetailsStruct.Description = 'The subtitle for the Icon Text Block.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);		

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Description';
		fieldDetailsStruct.Description = 'The Description for the Icon Text Block.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'LinkURL';
		fieldDetailsStruct.Description = 'The URL for the link associated with the link or button.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'LinkText';
		fieldDetailsStruct.Description = 'The text that will render for the Link or Button.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		return fieldMappingArray;
	</cfscript>
</cffunction>
