<!--- 
	csw-icontextblock.cfc
	
	This is the core rendering for the icon-textblock widget.  
	The csw-iconcountertextblock widget uses the same rendering.
		
	Version: [2020-10-01 01:30PM]
--->

<cfcomponent displayname="csw-icontextblock" output="false" hint="Common Icon Textblock Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			// load Resources
			Server.CommonSpot.udf.resources.loadResources("JQuery");
			Server.CommonSpot.udf.resources.loadResources("Bootstrap");
			Server.CommonSpot.udf.resources.loadResources("FontAwesome");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");	
			Server.CommonSpot.udf.resources.loadResources("cswButtonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswCardStyles");	
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");	
			Server.CommonSpot.udf.resources.loadResources("cswIconTextblock");
			
			// If a custom stylesheets have been registered, go ahead and load them.
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswButtonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswButtonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCardStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCardStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswIconTextblock_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswIconTextblock_custom");
		</cfscript>
	</cffunction>
	
	
	<cffunction name="loadProperties" access="public" returntype="struct" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW IconTextblock";	
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			var i = 0;
			var devices = 'xxs,xs,sm,md,lg';
			var device = '';
			
			
			// Check for Props and Layout Binding
			if ( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller,showDataError=arguments.showDataError) )
				return props;
			
			// 
			// Get Metadata (Core Icon Text	block metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
		
			// Theme 
			props.themeClass = "";
			if( StructKeyExists(widgetData,"Theme") )
				props.themeClass = TRIM(widgetData.Theme);
			
			
			/****************************************/
			/**** LAYOUT & CARD PROPERTIES TAB ******/
			/****************************************/ 
			
			// Layout
			props.layout = getStandardField( widgetData=widgetData, field='layout', defValue='cswLayout-Stacked' );
			
			// Card
			props.card = getFieldSettings(widgetData=widgetData, field='Card', defShow='yes', defBgColor='white', defStyle='cswShadow-medium' );	

			/**********************************/
			/**** GENERAL PROPERTIES TAB ******/
			/**********************************/

			// Element (STRUCT) (bgcolor, padding)
			props.element = getFieldSettings(widgetData=widgetData, field='Overall' );				
			
			// Columns (STRUCT)
			props.Columns = getDeviceSizeOptions(widgetData=widgetData, field='Columns', defXSS='cswColXXS-1', defXS='cswColXS-1', defSM='cswColSM-2', defMD='cswColMD-3', defLG='cswColLG-4');		
			props.colClass = getDeviceSizeClassList(props.Columns);
		
			// Gutter Margin
			props.gutterMargin = getStandardField( widgetData=widgetData, field='Gutter_Margin', defValue='' );
			
			
			// Background Image
			props.backgroundImage = getStandardMetadataImageField( widgetData=widgetData, field='Background_Image', defValue='' );		
			
			// Background Image Focal Point
			props.backgroundImageFocalPoint = getStandardField( widgetData=widgetData, field='Background_Image_Focal_Point', defValue='' );		
			
			
			/*******************************/
			/**** ICON PROPERTIES TAB ******/ 
			/*******************************/
			
			// Icon_Style (needed for show/hide)
			props.iconStyle = getStandardField( widgetData=widgetData, field='Icon_Style', defValue='cswIconStyle-plain' );
		
			// Icon (STRUCT)(color, size)
			props.iconSettings = getFieldSettings(widgetData=widgetData, field='Icon', defShow='', defColor='orange', defBgColor='white', style='cswSize-2x' );	
		
			// Icon_BG_Color (STRUCT)
			props.iconBgColor = getFieldSettings(widgetData=widgetData, field='Icon_BG_Color', defShow='' );	
				
			// Icon Inner Spacing
			props.iconInnerSpacing = getStandardField( widgetData=widgetData, field='Icon_InnerSpacing', defValue='10' );	
		
			
			/**************************************/
			/**** TITLE & DESC PROPERTIES TAB *****/ 
			/**************************************/
			
			// Title (STRUCT)
			props.title = getFieldSettings(widgetData=widgetData, field='Title', defShow='1', defTag='p', defColor='orange', defSize='cswSize-large');		
			
			// Subtitle (STRUCT)
			props.Subtitle = getFieldSettings(widgetData=widgetData, field='Subtitle', defShow='1', defTag='p', defColor='black', defSize='cswSize-normal');		
		
			// Line (STRUCT)
			props.line = getFieldSettings(widgetData=widgetData, field='Line', defShow='0', defColor='black', defSize='1px', defStyle='20%');		
			
			// Description_Color 
			props.desc = getFieldSettings(widgetData=widgetData, field='Description', defShow='1', defColor='black' );		
		
			
			/**************************************/
			/**** LINK/BUTTON PROPERTIES TAB ******/
			/**************************************/ 	
			
			// Button Type
			props.buttonType = getStandardField( widgetData=widgetData, field='Button_Type', defValue='cswButton' );
		
			// Button (STRUCT)
			props.button = getFieldSettings(widgetData=widgetData, field='Button', defShow='0', defColor='white', defBgColor='black', defSize='cswBtnSize-normal', defStyle='cswBtn-filled');		
			
			// Button_Corners
			props.buttonCorners = getStandardField( widgetData=widgetData, field='Button_Corners', defValue='cswBtn-rounded' );
			
			// Link (STRUCT)
			props.link = getFieldSettings(widgetData=widgetData, field='Link', defShow='0', defColor='inherit', defSize='cswBtnSize-normal', defStyle='');		
			
			// Button Icon
			props.buttonIcon = getStandardField( widgetData=widgetData, field='Button_Icon', defValue='' );

			/* Overrides */
			if( props.layout eq 'cswLayout-TwoColumnWrap' )
			{
				for( i=1; i lte listLen(devices); i=i+1 )
				{
					device = listGetAt(devices,i);
					// ensure no more than 2 columns
					props.columns[device] = replaceNoCase( props.columns[device], '-4', '-2', 'all' );
					props.columns[device] = replaceNoCase( props.columns[device], '-3', '-2', 'all' );
				}
				// rebuild colClasses list
				props.colClass = getDeviceSizeClassList(props.Columns);
			}	
			
			if( props.iconStyle eq 'cswIconStyle-plain' )		
			{
				props.iconBgColor.bgColor = '';	
			}

			return props;
		</cfscript>
	</cffunction>


	
	<cffunction name="getItemData" access="public" output="Yes" returntype="struct">
		<cfargument name="valuesStruct" type="struct" required="Yes">
		
		<cfscript>
			var data = StructNew();
			var values = arguments.valuesStruct;
			
			// Icon 
			data.icon = getStandardField(widgetData=values, field='iconClass', defValue='');
			
			// Title
			data.title = getStandardField(widgetData=values, field='title', defValue='');	

			// LINK URL 	
			data.linkURL = getStandardCustomElementLinkField(widgetData=values, field='LinkURL', defValue='');

			// Link Url Target
			data.LinkTarget = getStandardCustomElementLinkFieldTarget(widgetData=values, field='LinkURL', defValue='');

			// Link Url OnClick
			data.onClick = getStandardCustomElementLinkFieldOnClick(widgetData=values, field='LinkURL', defValue='');

			// Link Text					
			data.linktext = getStandardField(widgetData=values, field='linktext', defValue='');	
				
			// Subtitle
			data.subtitle = getStandardField(widgetData=values, field='subtitle', defValue='');	

			// Description
			data.desc = getStandardField(widgetData=values, field='description', defValue='');	
			
			return data;
		</cfscript>
		
	</cffunction>
	
	
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;
			var items = ArrayNew(1);
			var totalItems = 0;
			var i = 0;
			var styleAttr = '';		
			var data = StructNew();	
			
			// Get Records from the Element as an Array of Structures
			if( StructKeyExists(elementData, 'PropertyValues') AND ArrayLen(elementData.PropertyValues) )
				items = elementData.PropertyValues;
	
			totalItems = ArrayLen(items);
			
			// Alter Columns if number of items less than columns
			props.Columns = alterColumnClasses(totalItems, props.Columns);
			props.colClass = getDeviceSizeClassList(props.Columns);
		</cfscript>
	 
		<cfif totalItems>
		
			<cfscript>	
				styleAttr = '';
				if( len(props.backgroundImage) )
					styleAttr = styleAttr & "background-image: url('#props.backgroundImage#'); background-repeat: no-repeat; background-size: cover;";
					
				if( structKeyExists(props,'element') )
				{
					if( len(props.element.padding) )
						styleAttr = styleAttr & 'padding: #props.element.padding#;';
					if( len(props.element.bgColor) )
						styleAttr = styleAttr & 'background-color: #props.element.bgColor#;';
				}
				
				if( len(styleAttr) )
					styleAttr = 'style="#styleAttr#"';
			</cfscript>
		
			<cfoutput>
			<div class="cswIconTextblock #props.layout# #props.gutterMargin# #props.backgroundImageFocalPoint# #props.themeClass#" #styleAttr#>
			</cfoutput>
		
				<cfscript>
					//
					// Loop over each record and output a 'card'
					//
					for( i=1; i lte totalItems; i=i+1 )
					{
						// Get Data
						data = getItemData(items[i].values); 
						
						// RENDER the Icon Textblock container
						renderIconTextBlock( props, data );
					}
				</cfscript>
		
			<cfoutput></div></cfoutput>
		
		</cfif>
	
	</cffunction>
	
	
	
	<!----------------------------------------
		Render Icon Textblock
	----------------------------------------->
	<cffunction name="renderIconTextBlock" access="public" output="Yes" returntype="void">
		<cfargument name="propsStruct" required="Yes" type="struct">
		<cfargument name="dataStruct" required="Yes" type="struct">
		
		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			var i = 0;
			var containerClasses = '#props.iconSettings.align#';
	
			var containerStyles = '';
			var icon_wrapperClasses = '#props.iconStyle#';
			var icon_wrapperStyles = '';
         var iconStyles = '';
			var descStyles = 'color:#props.desc.color#;';
			var cardClasses = '#props.colClass#';
			var cardStyles = '';
			
			// Set Card styles			
			// -------------------
			if( len(props.card.align) )	
				containerClasses = trim(containerClasses & ' #props.card.align#');
					
			if( len(props.card.style) AND props.card.style neq 'cswShadow-none' )
				cardClasses = cardClasses & ' #props.card.style#';
	

			if( StructKeyExists(props.card,'borderColor') AND len(props.card.borderColor) )	
			{
				if( len(props.card.size) eq 0 )				
					props.card.size = '1px';
				if( props.card.size != 'none' )				
					cardStyles = cardStyles & 'border:#props.card.size# solid #props.card.borderColor#;';
			}
			
			if( FindNoCase('edgeup', props.card.style)  
			    AND ( len(props.card.bgColor) EQ 0 OR props.card.bgColor EQ 'inherit') )
			{
				// force background to be white (if not set and turn up shadow is set)
				containerStyles = containerStyles & 'background-color:white;';
			}			
			else if( len(props.card.bgColor) )
				containerStyles = containerStyles & 'background-color:#props.card.bgColor#;';
							
			if( len(props.card.padding) )	
				containerStyles = containerStyles & 'padding:#props.card.padding#;';
	
			// Card with Stacked Above - Need extra margin above as we are moving icon up
			if( props.layout eq 'cswLayout-StackedAbove' )
				cardStyles = cardStyles & 'margin-top:#((props.IconInnerSpacing*3)+20)#px;"';
	
			
			// Set Icon Styles
			// ----------------
	
			// Icon Color
			if( len(props.iconSettings.color) )
				iconStyles = iconStyles & 'color:#props.iconSettings.color#;';
	
			if( props.iconStyle neq 'cswIconStyle-plain' )
			{
				// render padding around icon unless Plain
				if( Len(props.iconInnerSpacing) )
					icon_wrapperStyles = icon_wrapperStyles & 'padding:#props.iconInnerSpacing#px;';
	
				// Background Color
				if( len(props.iconBgColor.bgColor) )
					icon_wrapperStyles = icon_wrapperStyles & 'background-color:#props.iconBgColor.bgColor#;';
	
				// Border Color
				if( StructKeyExists(props.iconBgColor,'borderColor') AND len(props.iconBgColor.borderColor) )
				{
					icon_wrapperStyles = icon_wrapperStyles & 'border-color:#props.iconBgColor.borderColor#;';
	
					// Border Size
					if( len(props.iconBgColor.size) )
						icon_wrapperStyles = icon_wrapperStyles & 'border-width:#props.iconBgColor.size#; border-style:solid;';
				}
				
				// If Icon Stacked Above, and padding, need negative margin to move up
				if( props.layout eq 'cswLayout-StackedAbove' )
				{
					if( len(props.card.padding) )
					{
						pad = ListFirst(props.card.padding, ' ');
						if( val(pad) gt 0 )
							icon_wrapperStyles = icon_wrapperStyles & 'margin-top:-#pad#';
					}
				}				
			}
	
			cardStyles = 'style="#cardStyles#"';	
			containerStyles = 'style="#containerStyles#"';	
			icon_wrapperStyles = 'style="#icon_wrapperStyles#"';	
			iconStyles = 'style="#iconStyles#"';	
			descStyles = 'style="#descStyles#"';	
		</cfscript>
		
		<cfif props.layout eq 'cswLayout-Stacked' OR 
				props.layout eq 'cswLayout-StackedAbove' OR
				props.layout eq 'cswLayout-IconNextTo' OR 
				props.layout eq 'cswLayout-IconNextTo cswBorderOnBottom' OR 
				props.layout eq 'cswLayout-TwoColumnWrap'>		
				
			<cfoutput>
			<!--- IconBlock Container --->
			<div class="cswCard #cardClasses#" #cardstyles#>
			   <div class="cswIconTextblockContainer #containerClasses#" #containerStyles#>
			
					<!--- icon --->
			     	<div class="cswIconWrapper #icon_wrapperClasses#" #icon_wrapperStyles#>
						<i class="cswIcon #data.icon# #props.iconSettings.size#" #iconStyles#></i>	
					</div>
		
			      <div class="cswTextblock">
						<!--- Title --->
						#renderOther(data, props)#
						#renderTitle(data, props)#
						#renderSubtitle(data, props)#
						#renderSeparatorLine(data, props)#
						#renderDescription(data, props)#
						#renderButton(data, props)#
			      </div>
				</div>
			</div>
			</cfoutput>
			
		<cfelseif props.layout eq 'cswLayout-IconTitleOnSameLine'>
		
			<cfoutput>
			<!--- IconBlock Container --->
			<div class="cswCard #cardClasses#" #cardstyles#>
			   <div class="cswIconTextblockContainer #containerClasses#" #containerStyles#>
			
					<div class="cswSameLineWrapper">
						<!--- Icon --->
				     	<div class="cswIconWrapper #icon_wrapperClasses#" #icon_wrapperStyles#>
							<i class="cswIcon #data.icon# #props.iconSettings.size#" #iconStyles#></i>	
						</div>
	
						<!--- Title --->
						#renderTitle(data,props)#
					</div>
		
			      <div class="cswTextblock">
						#renderOther(data, props)#
						#renderSubtitle(data,props)#
						#renderSeparatorLine(data,props)#
						#renderDescription(data,props)#
						#renderButton(data,props)#				
			      </div>
				
				</div>
			</div>
			</cfoutput>
	
		<cfelse>
		
			<cfoutput><p>Invalid layout: #props.layout#</p></cfoutput>			
			
		</cfif>	
	
	</cffunction>


	<cffunction name="renderOther" access="public" returntype="void" output="Yes">
		<cfargument name="dataStruct" type="struct" required="yes">
		<cfargument name="propsStruct" type="struct" required="yes">
		
		<!--- do nothing here (override to add functionality) --->
		
	</cffunction>
	
	
	<cffunction name="renderButton" access="private" output="yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="yes">
		<cfargument name="propsStruct" type="struct" required="yes">
	
		
		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			
			data.icon = '';
			data.buttonIcon = '';
			super.renderButton(data,props);
		</cfscript>
	</cffunction>

</cfcomponent>